{-# LANGUAGE OverloadedStrings #-}

module HAL where

import Data.Default (def)
import Data.Text

import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)

import Servant.Client (BaseUrl(..), Scheme(..), ClientM, ClientError, runClientM, mkClientEnv)

import HAL.Client
import HAL.Doc.Corpus
import HAL.Doc.Struct

import Servant.API
import Data.Aeson

getMetadataWith :: Text -> Maybe Int -> IO (Either ClientError (Response Corpus))
getMetadataWith q l = do
  manager' <- newManager tlsManagerSettings
  runHalAPIClient $ search (Just requestedFields) [q] Nothing l Nothing

requestedFields :: Text
requestedFields = "docid,title_s,abstract_s,submittedDate_s,source_s,authFullName_s,authOrganism_s"

structFields :: Text
structFields = "docid,label_s,parentDocid_i"

runHalAPIClient :: (FromJSON doc, ToHttpApiData doc) =>
  ClientM (Response doc) -> IO (Either ClientError (Response doc))
runHalAPIClient cmd = do
  manager' <- newManager tlsManagerSettings
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.archives-ouvertes.fr" 443 "")

runStructureRequest :: Maybe Text -> IO (Either ClientError (Response Struct))
runStructureRequest rq =
  runHalAPIClient $ structure (Just structFields) rq (Just 10000)

runSearchRequest :: [Text] -> IO (Either ClientError (Response Corpus))
runSearchRequest rq =
  runHalAPIClient $ search (Just requestedFields) rq Nothing Nothing Nothing

generateRequestByStructID :: Text -> [Text] -> Text
generateRequestByStructID rq struct_ids =
  "text:" <>
  rq
  <> " AND "
  <> "structId_i:("
  <> (flattenPipe struct_ids)
  <> ")"

flattenPipe :: [Text] -> Text
flattenPipe (x:[]) = x
flattenPipe (x:xs) = x <> " || " <> flattenPipe xs
